@@ -7,7 +7,7 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.31';
+our $VERSION = '0.35';
 
 #Variables global to the package	
 my $maximize_total_benefit;
@@ -22,16 +22,16 @@ my ( $array1_size, $array2_size, $min_size, $max_size, $original_max_size );
 my ( %assignned_object, %assignned_person, %objects_desired_by_this, %price_object );
 my %index_correlation;
 
-sub auction { #                        => default values
-	my %args = ( matrix_ref            => undef,     # reference to array: matrix N x M			                                     
-				maximize_total_benefit => 0,         # 0: minimize_total_benefit ; 1: maximize_total_benefit
-				inicial_stepsize       => undef,     # auction algorithm terminates with a feasible assignment if the problem data are integer and stepsize < 1/min(N,M)				
-				inicial_price          => 0,
-				verbose                => 3,         # level of verbosity, 0: quiet; 1, 2, 3, 4, 9, 10: debug information.				
-				@_,                                  # argument pair list goes here
+sub auction { #							=> default values
+	my %args = ( matrix_ref				=> undef,     # reference to array: matrix N x M			                                     
+				maximize_total_benefit  => 0,         # 0: minimize_total_benefit ; 1: maximize_total_benefit
+				inicial_stepsize        => undef,     # auction algorithm terminates with a feasible assignment if the problem data are integer and stepsize < 1/min(N,M)				
+				inicial_price           => 0,
+				verbose                 => 3,         # level of verbosity, 0: quiet; 1, 2, 3, 4, 9, 10: debug information.				
+				@_,                                   # argument pair list goes here
 				);
        
-	my @matrix_input = @{$args{matrix_ref}};         # Input: Reference to the input matrix (NxM) = $min_size x $max_size
+	my @matrix_input = @{$args{matrix_ref}};          # Input: Reference to the input matrix (NxM) = $min_size x $max_size
    
 	$array1_size = $#matrix_input + 1;
 	$array2_size = $#{$matrix_input[0]} + 1;
@@ -81,7 +81,7 @@ sub auction { #                        => default values
 	   $epsilon = $args{inicial_stepsize} if ( defined $args{inicial_stepsize} );
 	   $epsilon = 1/(1+$min_size) if ($epsilon < 1/$min_size);
     
-	my ( @assignment, @prices );
+	my ( @assignment, @prices, %same_benefit );
 	my $feasible_assignment_condition = 0;
 
 	# The preceding observations suggest the idea of epsilon-scaling, which consists of applying the algorithm several times, 
@@ -91,12 +91,14 @@ sub auction { #                        => default values
    
 		%assignned_object = ();
 		%assignned_person = ();
+		%objects_desired_by_this = ();
 		$iter_count_local = 0;
 
 		while ( (scalar keys %assignned_person) < $max_size ){ # while there is at least one element not assigned.
          
 			$iter_count_global++;
-			$iter_count_local++;		
+			$iter_count_local++;
+			
 			auctionRound( \@matrix, $epsilon, $args{verbose} );
 		 
 			if ( $args{verbose} >= 10 ){
@@ -115,6 +117,7 @@ sub auction { #                        => default values
 					}
 					if ($i >= 0){ print "]\n"; } else{ print "\n\n"; }
 				}
+				print "\n";		
 			}		 
 		}
 		
@@ -159,7 +162,7 @@ sub auction { #                        => default values
 	}}   
    
 	if ( $args{verbose} >= 10 ){
-		printf "\n\$optimal_benefit = $optimal_benefit ; \$iter_count_global = $iter_count_global ; \$epsilon = %.4g ; \@output_index = (@output_index) ; \@assignment = (@assignment) ; \@prices = (@prices) \n", $epsilon; 
+		printf "\$optimal_benefit = $optimal_benefit ; \$iter_count_global = $iter_count_global ; \$epsilon = %.4g ; \@output_index = (@output_index) ; \@assignment = (@assignment) ; \@prices = (@prices) \n", $epsilon; 
 	}   
 	print_screen_messages( \@matrix, \@matrix_index, \@matrix_input, \@output_index, $optimal_benefit, $args{verbose}, $epsilon ) ;
        
@@ -326,28 +329,28 @@ sub print_screen_messages {
 	  
       my %orderly_solution;
       for my $i ( 0 .. $original_max_size - 1 ){
-         my $j = $output_index[$i];
-         my $weight = $max_matrix_value;
-         $weight = $matrix_input[$i]->[$j] if ( defined $matrix_input[$i] and defined $matrix_input[$i]->[$j] ); # condition for valid solution
+		my $j = $output_index[$i];
+		my $weight = $max_matrix_value;
+		$weight = $matrix_input[$i]->[$j] if ( defined $matrix_input[$i] and defined $matrix_input[$i]->[$j] ); # condition for valid solution
          
-         $orderly_solution{ $weight } { $i } { 'index_array1' } = $i;
-         $orderly_solution{ $weight } { $i } { 'index_array2' } = $j;		 
+		$orderly_solution{ $weight } { $i } { 'index_array1' } = $i;
+		$orderly_solution{ $weight } { $i } { 'index_array2' } = $j;		 
       }
 
       print "\n Pairs (in ascending order of matrix values):\n"; 
 
-      my $sum_matrix_value = 0;
+	  my $sum_matrix_value = 0;
       foreach my $matrix_value ( sort { $a <=> $b }  keys %orderly_solution ){
       foreach my $k ( sort { $a <=> $b } keys %{$orderly_solution{$matrix_value}} ){
      
-	     my $index_array1 = $orderly_solution{ $matrix_value } { $k } { 'index_array1' };
-         my $index_array2 = $orderly_solution{ $matrix_value } { $k } { 'index_array2' };
+		my $index_array1 = $orderly_solution{ $matrix_value } { $k } { 'index_array1' };
+		my $index_array2 = $orderly_solution{ $matrix_value } { $k } { 'index_array2' };
 	  
-	     $sum_matrix_value += $matrix_value if ( defined $matrix_input[$index_array1] and defined $matrix_input[$index_array1]->[$index_array2] );
+		$sum_matrix_value += $matrix_value if ( defined $matrix_input[$index_array1] and defined $matrix_input[$index_array1]->[$index_array2] );
 	  
-	     my $weight = ( defined $matrix_input[$index_array1] and defined $matrix_input[$index_array1]->[$index_array2] ) ? sprintf( "%${matrix_spaces}.${decimals}f", $matrix_value ) : ' ' x $matrix_spaces ;
+		my $weight = ( defined $matrix_input[$index_array1] and defined $matrix_input[$index_array1]->[$index_array2] ) ? sprintf( "%${matrix_spaces}.${decimals}f", $matrix_value ) : ' ' x $matrix_spaces ;
 	  
-         printf( "   indices ( %${index_length}d, %${index_length}d ), matrix value = $weight ; sum of values = %${matrix_spaces}.${decimals}f \n", $index_array1, $index_array2, $sum_matrix_value );
+		printf( "   indices ( %${index_length}d, %${index_length}d ), matrix value = $weight ; sum of values = %${matrix_spaces}.${decimals}f \n", $index_array1, $index_array2, $sum_matrix_value );
       }}	  
    }
 
@@ -361,11 +364,11 @@ sub get_matrix_info {
    for my $i ( 0 .. $#matrix ) {
    for my $j ( 0 .. $#{$matrix[$i]} ) {
       
-	  my $char_number = length( $matrix[$i]->[$j] ); # count the number of characters
-      $matrix_spaces = $char_number if ( (not defined $matrix_spaces) || ($char_number > $matrix_spaces) );
+		my $char_number = length( $matrix[$i]->[$j] ); # count the number of characters
+		$matrix_spaces = $char_number if ( (not defined $matrix_spaces) || ($char_number > $matrix_spaces) );
 	  
-      $max_matrix_value = $matrix[$i]->[$j] if ( (not defined $max_matrix_value) || ($matrix[$i]->[$j] > $max_matrix_value) );	  
-	  $min_matrix_value = $matrix[$i]->[$j] if ( (not defined $min_matrix_value) || ($matrix[$i]->[$j] < $min_matrix_value) );
+		$max_matrix_value = $matrix[$i]->[$j] if ( (not defined $max_matrix_value) || ($matrix[$i]->[$j] > $max_matrix_value) );	  
+		$min_matrix_value = $matrix[$i]->[$j] if ( (not defined $min_matrix_value) || ($matrix[$i]->[$j] < $min_matrix_value) );
    }}
    
    $decimals = length(($max_matrix_value =~ /[,.](\d+)/)[0]); # counting the number of digits after the decimal point
@@ -383,9 +386,9 @@ sub get_matrix_info {
       for my $i ( 0 .. $#matrix ) {
       for my $j ( 0 .. $#{$matrix[$i]} ) {
 	     
-		 $matrix[$i]->[$j] = $matrix[$i]->[$j] - $min_matrix_value ;
+		$matrix[$i]->[$j] = $matrix[$i]->[$j] - $min_matrix_value ;
 		 
-	     #$matrix[$i]->[$j] = 99 * ( $matrix[$i]->[$j] - $min_matrix_value ) / $range; # new scale: Min = 0 <---> Max = 99
+		#$matrix[$i]->[$j] = 99 * ( $matrix[$i]->[$j] - $min_matrix_value ) / $range; # new scale: Min = 0 <---> Max = 99
 
       }}   
  	  
@@ -394,9 +397,9 @@ sub get_matrix_info {
       for my $i ( 0 .. $#matrix ) {
       for my $j ( 0 .. $#{$matrix[$i]} ) {   
 	     
-		 $matrix[$i]->[$j] = $max_matrix_value - $matrix[$i]->[$j] ;
+		$matrix[$i]->[$j] = $max_matrix_value - $matrix[$i]->[$j] ;
 		 
-		 #$matrix[$i]->[$j] = 99 * ( $max_matrix_value - $matrix[$i]->[$j] ) / $range;
+		#$matrix[$i]->[$j] = 99 * ( $max_matrix_value - $matrix[$i]->[$j] ) / $range; # new scale: Min = 0 <---> Max = 99
       }}	  
    }
 }
@@ -404,10 +407,10 @@ sub get_matrix_info {
 sub auctionRound {
 	my ( $matrix_ref, $epsilon, $verbose ) = @_;
 	my @matrix = @$matrix_ref;
-	my %info;
+	my %info; my %seen_bid;
 
 	if ( $verbose >= 10 ){
-		print "\n Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
+		print " Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
 		foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 			my $object = $assignned_person{$person};
 			printf " \$assignned_person{%2s} --> object %2s --> \$price_object{%2s} = $price_object{$object} \n", $person, $object, $object;
@@ -433,18 +436,19 @@ sub auctionRound {
 			##  |  ...
 			##  i  person (N - 1)    price_i_0	
 			
-			my ( $optObjForPersonI, $SecOptObjForPersonI, $ThdOptObjForPersonI, $FthOptObjForPersonI );			
-			my ( $optValForPersonI, $SecOptValForPersonI, $ThdOptValForPersonI, $FthOptValForPersonI ) = ( -1 * $max_matrix_value, -1 * $max_matrix_value, -1 * $max_matrix_value, -1 * $max_matrix_value );
+			my ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI, $Opt04ObjForPersonI, $Opt05ObjForPersonI, $Opt06ObjForPersonI );
+			my ( $Opt01ValForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $Opt04ValForPersonI, $Opt05ValForPersonI, $Opt06ValForPersonI );
+			my ( $bid01ForPersonI, $bid02ForPersonI );
             
 			# "The set of objects to which each person i has been assigned in the last few assignments does not change much as the algorithm proceeds."
 			# Each person is assigned to just a relatively small number of objects during the execution of the auctionRound.
 			# My implementation is different from Libor Bus and Pavel Tvrdik: 'Towards auction algorithms for large dense assignment problems' (see reference 2).
-			# My working sets @objects_with_greater_benefits are updated dynamically by %objects_desired_by_this hash.
+			# My working sets @objects_with_greater_benefits are updated dynamically by %objects_desired_by_this.						
 			
-			my @objects_with_greater_benefits = keys %{$objects_desired_by_this{$person}};
+			my @objects_with_greater_benefits = keys %{$objects_desired_by_this{$person}}; # sort { $a <=> $b } keys %{$objects_desired_by_this{$person}}	            			
 			my @not_assignned_objects = grep { not defined $assignned_object{$_} } @objects_with_greater_benefits;
-			
-			for my $object ( @not_assignned_objects > 1 ? @objects_with_greater_benefits : ( 0 .. $max_size - 1 ) )		
+
+			for my $object ( @not_assignned_objects > 0 ? @objects_with_greater_benefits : ( 0 .. $max_size - 1 ) )		
 			#for my $object ( 0 .. $max_size - 1 )			
 			{				
 				$seen_ghost++ if ( not defined $matrix[$person] and $min_size < $max_size );
@@ -453,67 +457,107 @@ sub auctionRound {
 				
 				my $curVal = $matrix_value - $price_object{$object};
 				
-				if ( $curVal > $optValForPersonI )
-				{					
-					$FthOptValForPersonI = $ThdOptValForPersonI;
-					$FthOptObjForPersonI = $ThdOptObjForPersonI;
-					$ThdOptValForPersonI = $SecOptValForPersonI;
-					$ThdOptObjForPersonI = $SecOptObjForPersonI;
-					$SecOptValForPersonI = $optValForPersonI;
-					$SecOptObjForPersonI = $optObjForPersonI;
-					$optValForPersonI = $curVal;
-					$optObjForPersonI = $object;
-				}
-				elsif ( $curVal > $SecOptValForPersonI )
+				if ( (not defined $Opt01ValForPersonI) || ($curVal > $Opt01ValForPersonI) )
 				{
-					$FthOptValForPersonI = $ThdOptValForPersonI;
-					$FthOptObjForPersonI = $ThdOptObjForPersonI;
-					$ThdOptValForPersonI = $SecOptValForPersonI;
-					$ThdOptObjForPersonI = $SecOptObjForPersonI;
-					$SecOptValForPersonI = $curVal;
-					$SecOptObjForPersonI = $object;
+					$Opt06ValForPersonI = $Opt05ValForPersonI;
+					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
+					$Opt05ValForPersonI = $Opt04ValForPersonI;
+					$Opt05ObjForPersonI = $Opt04ObjForPersonI;
+					$Opt04ValForPersonI = $Opt03ValForPersonI;
+					$Opt04ObjForPersonI = $Opt03ObjForPersonI;
+					$Opt03ValForPersonI = $Opt02ValForPersonI;
+					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
+					$Opt02ValForPersonI = $Opt01ValForPersonI;
+					$Opt02ObjForPersonI = $Opt01ObjForPersonI;
+					$Opt01ValForPersonI = $curVal;
+					$Opt01ObjForPersonI = $object;
 				}
-				elsif ( $curVal > $ThdOptValForPersonI )
+				elsif ( (not defined $Opt02ValForPersonI) || ($curVal > $Opt02ValForPersonI) )
 				{
-					$FthOptValForPersonI = $ThdOptValForPersonI;
-					$FthOptObjForPersonI = $ThdOptObjForPersonI;
-					$ThdOptValForPersonI = $curVal;
-					$ThdOptObjForPersonI = $object;
+					$Opt06ValForPersonI = $Opt05ValForPersonI;
+					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
+					$Opt05ValForPersonI = $Opt04ValForPersonI;
+					$Opt05ObjForPersonI = $Opt04ObjForPersonI;
+					$Opt04ValForPersonI = $Opt03ValForPersonI;
+					$Opt04ObjForPersonI = $Opt03ObjForPersonI;
+					$Opt03ValForPersonI = $Opt02ValForPersonI;
+					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
+					$Opt02ValForPersonI = $curVal;
+					$Opt02ObjForPersonI = $object;
 				}
-				elsif ( $curVal > $FthOptValForPersonI )
+				elsif ( (not defined $Opt03ValForPersonI) || ($curVal > $Opt03ValForPersonI) )
 				{
-					$FthOptValForPersonI = $curVal;
-					$FthOptObjForPersonI = $object;
+					$Opt06ValForPersonI = $Opt05ValForPersonI;
+					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
+					$Opt05ValForPersonI = $Opt04ValForPersonI;
+					$Opt05ObjForPersonI = $Opt04ObjForPersonI;
+					$Opt04ValForPersonI = $Opt03ValForPersonI;
+					$Opt04ObjForPersonI = $Opt03ObjForPersonI;
+					$Opt03ValForPersonI = $curVal;
+					$Opt03ObjForPersonI = $object;
+				}
+				elsif ( (not defined $Opt04ValForPersonI) || ($curVal > $Opt04ValForPersonI) )
+				{
+					$Opt06ValForPersonI = $Opt05ValForPersonI;
+					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
+					$Opt05ValForPersonI = $Opt04ValForPersonI;
+					$Opt05ObjForPersonI = $Opt04ObjForPersonI;
+					$Opt04ValForPersonI = $curVal;
+					$Opt04ObjForPersonI = $object;
+				}
+				elsif ( (not defined $Opt05ValForPersonI) || ($curVal > $Opt05ValForPersonI) )
+				{
+					$Opt06ValForPersonI = $Opt05ValForPersonI;
+					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
+					$Opt05ValForPersonI = $curVal;
+					$Opt05ObjForPersonI = $object;
+				}
+				elsif ( (not defined $Opt06ValForPersonI) || ($curVal > $Opt06ValForPersonI) )
+				{
+					$Opt06ValForPersonI = $curVal;
+					$Opt06ObjForPersonI = $object;
 				}
 				
 				if ( $verbose >= 10 ){
-					printf " personI = %2s ; objectJ = %2s ; \$curVal %10.5f = \$matrix[%2s][%2s] %2.0f - \$price_object{%2s} %10.5f \n", $person, $object, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};
+					printf " personI = %2s ; objectJ = %2s ; \$curVal %10.5f = \$matrix[%2s][%2s] %3.0f - \$price_object{%2s} %10.5f \n", $person, $object, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};
 				}				
 			}
 			
-			my $bidForPersonI = $optValForPersonI - $SecOptValForPersonI + $epsilon;
+			$bid01ForPersonI = $Opt01ValForPersonI - $Opt02ValForPersonI + $epsilon;
 			
 			# Stores the bidding info for future use
-			$info{$optObjForPersonI}{$bidForPersonI} = $person if ( not defined $info{$optObjForPersonI}{$bidForPersonI} ); # get only one person with this bid
+			$info{$Opt01ObjForPersonI}{$bid01ForPersonI} = $person if ( not defined $info{$Opt01ObjForPersonI}{$bid01ForPersonI} ); # get only one person with this bid			
+
+			if ( $max_size >= 10 ){
 			
-			$objects_desired_by_this{$person}{$optObjForPersonI}    = 1; # information about the most desired objects
-			$objects_desired_by_this{$person}{$SecOptObjForPersonI} = 1 if (defined $SecOptObjForPersonI);
-			$objects_desired_by_this{$person}{$ThdOptObjForPersonI} = 1 if (defined $ThdOptObjForPersonI);
-			$objects_desired_by_this{$person}{$FthOptObjForPersonI} = 1 if (defined $FthOptObjForPersonI);			
+				$bid02ForPersonI = $Opt02ValForPersonI - $Opt03ValForPersonI + $epsilon; # calculate $bid02ForPersonI to try to predict the values of the next round
+				$bid02ForPersonI = 2 * $epsilon if ( $Opt01ValForPersonI > ($Opt02ValForPersonI + $epsilon) or $Opt02ValForPersonI > ($Opt03ValForPersonI + $epsilon) );
+			
+				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );
+				
+				$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = 1; # information about the most desired objects
+				$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = 1 if (defined $Opt02ObjForPersonI);
+				$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = 1 if (defined $Opt03ObjForPersonI);
+				$objects_desired_by_this{$person}{$Opt04ObjForPersonI} = 1 if (defined $Opt04ObjForPersonI);
+				$objects_desired_by_this{$person}{$Opt05ObjForPersonI} = 1 if (defined $Opt05ObjForPersonI);
+				$objects_desired_by_this{$person}{$Opt06ObjForPersonI} = 1 if (defined $Opt06ObjForPersonI);								
+			}
 			
 			if ( $verbose >= 10 ){
-				my @array = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};				
-				printf "<> PersonI = %2s ; \@objects_with_greater_benefits = (@objects_with_greater_benefits) ; object eligible to bid --> \@not_assignned_objects = (@not_assignned_objects) : %2s > 1 ? \n", $person, scalar @not_assignned_objects;
+				my @array = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};
+
+				printf "<> PersonI = %2s ; \@objects_with_greater_benefits = (@objects_with_greater_benefits) ; object eligible to bid --> \@not_assignned_objects = (@not_assignned_objects) : %2s > 0 ? \n", $person, scalar @not_assignned_objects;
 				printf "<> PersonI = %2s ; objects desired by this person = (@array) \n", $person;
-				printf "<> PersonI = %2s chose ObjectJ = %2s ; \$bidForPersonI %.5f = \$optValForPersonI %.5f - \$SecOptValForPersonI %.5f + \$epsilon %.5f \n", $person, $optObjForPersonI, $bidForPersonI, $optValForPersonI, $SecOptValForPersonI, $epsilon;							    								
+				printf "<> PersonI = %2s chose ObjectJ = %2s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
+				printf "<> PersonI = %2s chose ObjectJ = %2s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
 			}
 		}
-	}	
+	}		
 	
 	foreach my $object ( keys %info ){                                 # for each object, choose only the first bid.
 	foreach my $bid    ( sort { $b <=> $a } keys %{$info{$object}} ){  # descending order!!! --> the first bid is the highest bid.
 	
-		my $person       = $info{$object}{$bid};
+		my $person       = $info{$object}{$bid};		
 		my $other_person = $assignned_object{$object}; # Find the other person who has objectJ and make them unassigned
 	   	   	   
 		if ( defined $other_person ) {
@@ -534,7 +578,7 @@ sub auctionRound {
 		$assignned_person{$person} = $object;
 		$assignned_object{$object} = $person;
 	   
-		$price_object{$object} += $bid;	
+		$price_object{$object} += $bid;
 	   
 		if ( $verbose >= 10 ){
 			printf " --> Assigning to personI = %2s the objectJ = %2s with highestBidForJ = %10.5f and update the price vector ; \$assignned_person{%2s} = %2s ; \$price_object{%2s} = %.5f \n", $person, $object, $bid, $person, $assignned_person{$person}, $object, $price_object{$object};	
@@ -542,7 +586,7 @@ sub auctionRound {
 	   
 		last;  # next object, choose only the highest bid for each object
 	}}
-
+	
 	if ( $verbose >= 10 ){
 		print "\n Final: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
 		foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
@@ -729,13 +773,15 @@ __END__
    my $j = $assignement_ref->{$i};   
    ...
  }
- 
+
  my $sum = 0;
  for my $i ( 0 .. $#{$output_index_ref} ){
 	my $j = $output_index_ref->[$i];   
-	my $value = $input_matrix[$i]->[$j];  
-	$sum += $value;
-	printf " Auction Algorithm, output index --> \$i = %3d ; \$j = %3d ; \$value = %5s ; \$sum = %5s \n", $i, $j, $value, $sum;
+	my $value = $input_matrix[$i]->[$j];
+	$sum += $value if (defined $value);
+	
+	$value = defined $value ? sprintf( "%6s", $value ) : ' ' x 6 ; # %6s  
+	printf " Auction Algorithm, output index --> \$i = %3d ; \$j = %3d ; \$value = $value ; \$sum = %8s \n", $i, $j, $sum;
  }
  
 =head1 OPTIONS
@@ -783,7 +829,7 @@ __END__
 =head1 AUTHOR
 
     Claudio Fernandes de Souza Rodrigues
-	March 25, 2018
+	March 31, 2018
 	Sao Paulo, Brasil
 	claudiofsr@yahoo.com
 	
